//+------------------------------------------------------------------+
//|                                              Cronex DeMarker.mq4 |
//|                                        Copyright  2007, Cronex. |
//|                                       http://www.metaquotes.net/ |
//+------------------------------------------------------------------+
#property  copyright "Copyright  2007, Cronex"
#property  link      "http://www.metaquotes.net/"

#property  indicator_separate_window
#property  indicator_buffers 3
#property  indicator_color1  Silver
#property  indicator_color2  Blue
#property  indicator_color3  Red

//
//
//
//
//

extern int    DeMarker         = 25;
extern int    FastMA           = 14;
extern int    SlowMA           = 25;
extern bool   alertsOn         = false;
extern bool   alertsOnCurrent  = true;
extern bool   alertsMessage    = true;
extern bool   alertsSound      = false;
extern bool   alertsEmail      = false;
extern bool   alertsNotify     = false;
extern string soundFile        = "alert2.wav";
extern bool   ShowArrows       = true;
extern string arrowsIdentifier = "cronexDemark Arrows1";
extern double arrowsUpperGap   = 1.0;
extern double arrowsLowerGap   = 1.0;
extern color  arrowsUpColor    = LimeGreen;
extern color  arrowsDnColor    = Red;
extern int    arrowsUpCode     = 241;
extern int    arrowsDnCode     = 242;

//
//
//
//
//

double DeMarkerBuffer[];
double FastMABuffer[];
double SlowMABuffer[];
double trend[];

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
//
//

int init()
{
   IndicatorBuffers(4);
   SetIndexBuffer(0,DeMarkerBuffer); SetIndexLabel(0,"DeMarker");
   SetIndexBuffer(1,FastMABuffer);   SetIndexLabel(1,"Fast MA");
   SetIndexBuffer(2,SlowMABuffer);   SetIndexLabel(2,"Slow MA");
   SetIndexBuffer(3,trend);
   IndicatorShortName("Cronex DeMarker");
return(0);
}

int deinit() 
{  
   deleteArrows(); 
return(0); 
}

//+------------------------------------------------------------------+
//| Cronex DeMarker                                                  |
//+------------------------------------------------------------------+
//
//

int start()
{
   int i,counted_bars = IndicatorCounted();
   if (counted_bars < 0) return(-1);
   if (counted_bars > 0) counted_bars--;
         int limit = MathMin(Bars-counted_bars,Bars-1);
   
   //
   //
   //
   //
   //
   
   for(i = limit; i>=0; i--) DeMarkerBuffer[i]=iDeMarker(NULL,0,DeMarker,i);
   for(i = limit; i>=0; i--) 
   {  
      FastMABuffer[i]=iMAOnArray(DeMarkerBuffer,0,FastMA,0,MODE_LWMA,i);
      SlowMABuffer[i]=iMAOnArray(DeMarkerBuffer,0,SlowMA,0,MODE_LWMA,i);
      trend[i] = trend[i+1];
      if (DeMarkerBuffer[i]>SlowMABuffer[i] && DeMarkerBuffer[i]>FastMABuffer[i]) trend[i] = 1;
      if (DeMarkerBuffer[i]<SlowMABuffer[i] && DeMarkerBuffer[i]<FastMABuffer[i]) trend[i] =-1;
     
      //
      //
      //
      //
      //
     
      if (ShowArrows)
      {
        deleteArrow(Time[i]);
        if (trend[i] != trend[i+1])
        {
          if (trend[i] == 1)  drawArrow(i,arrowsUpColor,arrowsUpCode,false);
          if (trend[i] ==-1)  drawArrow(i,arrowsDnColor,arrowsDnCode, true);
        }
      }
   }
   
   //
   //
   //
   //
   //
      
   if (alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1;
      if (trend[whichBar] != trend[whichBar+1])
      if (trend[whichBar] == 1)
            doAlert("crossed up");
      else  doAlert("crossed down");       
   }       
return(0);
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

void doAlert(string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
      if (previousAlert != doWhat || previousTime != Time[0]) {
          previousAlert  = doWhat;
          previousTime   = Time[0];

          //
          //
          //
          //
          //

          message =  StringConcatenate(Symbol()," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," Cronex DeMarker ",doWhat);
             if (alertsMessage) Alert(message);
             if (alertsNotify)  SendNotification(message);
             if (alertsEmail)   SendMail(StringConcatenate(Symbol()," Cronex DeMarker "),message);
             if (alertsSound)   PlaySound(soundFile);
      }
}

//
//
//
//
//

void drawArrow(int i,color theColor,int theCode,bool up)
{
   string name = arrowsIdentifier+":"+Time[i];
   double gap  = iATR(NULL,0,20,i);   
   
      //
      //
      //
      //
      //
      
      ObjectCreate(name,OBJ_ARROW,0,Time[i],0);
         ObjectSet(name,OBJPROP_ARROWCODE,theCode);
         ObjectSet(name,OBJPROP_COLOR,theColor);
         if (up)
               ObjectSet(name,OBJPROP_PRICE1,High[i] + arrowsUpperGap * gap);
         else  ObjectSet(name,OBJPROP_PRICE1,Low[i]  - arrowsLowerGap * gap);
}

//
//
//
//
//

void deleteArrows()
{
   string lookFor       = arrowsIdentifier+":";
   int    lookForLength = StringLen(lookFor);
   for (int i=ObjectsTotal()-1; i>=0; i--)
   {
      string objectName = ObjectName(i);
         if (StringSubstr(objectName,0,lookForLength) == lookFor) ObjectDelete(objectName);
   }
}

//
//
//
//
//

void deleteArrow(datetime time)
{
   string lookFor = arrowsIdentifier+":"+time; ObjectDelete(lookFor);
}





